﻿using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Design;
using Microsoft.EntityFrameworkCore.Metadata;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace TTEFCore5
{
    public class AppDbContext : DbContext
    {
        public DbSet<Person> People { get; set; }

        public AppDbContext()
        {
        }

        public AppDbContext(DbContextOptions<AppDbContext> options) : base(options)
        {
        }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {

            modelBuilder.Entity<Person>().HasKey(x => x.Id);

            modelBuilder.Entity<Person>().Property(x => x.ValidFromUTC)
                                       .IsRequired()
                                       .ValueGeneratedOnAddOrUpdate()
                                       .HasDefaultValueSql("SYSUTCDATETIME()");

            modelBuilder.Entity<Person>().Property(x => x.ValidToUTC)
                                       .IsRequired()
                                       .ValueGeneratedOnAddOrUpdate()
                                       .HasDefaultValueSql("CONVERT(DATETIME2, '9999-12-31 23:59:59.9999999')");

            base.OnModelCreating(modelBuilder);
        }

        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
        {
            optionsBuilder.UseSqlServer("Data Source=(local);Initial Catalog=TTEFCore5;Integrated Security=True");
            optionsBuilder.LogTo(m => Debug.WriteLine(m));
            base.OnConfiguring(optionsBuilder);
        }
    }


    public class DesignTimeAppDbContextFactory : IDesignTimeDbContextFactory<AppDbContext>
    {
        public AppDbContext CreateDbContext(string[] args)
        {
            var optionsBuilder = new DbContextOptionsBuilder<AppDbContext>();
            optionsBuilder.UseSqlServer("Data Source=(local);Initial Catalog=TTEFCore5;Integrated Security=True");
            return new AppDbContext(optionsBuilder.Options);
        }
    }
}
